#include <sys/socket.h>
#include <sys/un.h>
#include <netinet/in.h>
#include <netdb.h>
#include <stddef.h>
#include <stdlib.h>
#include <errno.h>

#include "common.h"

/* bind for internet
 * 
 * ADDRESS is interpreted as a name to look up in the services database
 */
static int 
afinet_bind (int fd, const char *protocol, const char *address, cenv env)
{
  struct sockaddr_in address_structure;

  address_structure.sin_family = AF_INET;

  {
    char *tail;

    errno = 0;
    address_structure.sin_port = htons(strtoul (address, &tail, 0));

    if (*tail || errno)
      {
	struct servent *service_entry;

	service_entry = getservbyname (address, protocol);
	if (!service_entry)
	  return -1;

	address_structure.sin_port = service_entry->s_port;
      }
  }

  address_structure.sin_addr.s_addr = INADDR_ANY;

  return bind (fd, &address_structure, sizeof address_structure);
}

/* Bind for unix domain
 * 
 * ADDRESS is simply a path name.
 *
 */
static int 
afunix_bind (int fd, const char *protocol, const char *address, cenv env)
{
  struct sockaddr_un *address_structure;
  size_t address_size;

  address_size = strlen (address) + 1 + offsetof (struct sockaddr_un, sun_path);
  address_structure = alloca (address_size);
  strcpy (address_structure->sun_path, address);

  return bind (fd, address_structure, address_size);
}

const struct protocol protocols_table[] =
{
  {"udp", PF_INET, SOCK_DGRAM, IPPROTO_UDP, &afinet_bind},
  {"tcp", PF_INET, SOCK_STREAM, IPPROTO_TCP, &afinet_bind},
  {"unix", PF_UNIX, SOCK_STREAM, 0, &afunix_bind},
  {0, 0, 0, 0, 0}
};
